import { MonacoEventsDefinition, EventsDelegatorOptions, BaseEventsDelegator } from '@react-monaco/core';
import { Registry, IOnigLib } from 'vscode-textmate';

declare const setupTextmate: (data: Partial<{
    onigurumaWasmUrl: URL;
    baseUrl: URL;
    languages: string;
}>) => void;
declare const tmConfig: <K extends "onigurumaWasmUrl" | "baseUrl" | "languages">(key: K) => {
    onigurumaWasmUrl: URL;
    baseUrl: URL;
    languages: string;
}[K];
declare const isTmSupportLanguage: (id: string) => boolean;

type TextmateActiveLanguage = {
    isActive: boolean;
    languageId: string;
    extname?: string;
    timestamp: number;
};
type TextmateProvider = {
    url: string | URL;
    languageId: string;
    scopeName?: string;
};
type TextmateProviderCallback = (params: MonacoEventsDefinition['prepareModel']) => TextmateProvider | undefined;
type TextmateFilterCodeSetCallback = (code: TextmateCodeSet) => TextmateCodeSet;
type TextmateInjectionProps = EventsDelegatorOptions & {
    onigurumaWasmUrl?: string | URL;
    provider?: TextmateProviderCallback;
    baseUrl?: string | URL;
    onChange?: (state: TextmateActiveLanguage) => void;
    filter?: TextmateFilterCodeSetCallback;
};
type TextmateScope = {
    scopeName: string;
    tmName: string;
};
type TextmateCodeSet = {
    isWired: boolean;
    extname?: string;
    language?: monaco.languages.ILanguageExtensionPoint;
    languageId: string;
    provider?: TextmateProvider;
} & TextmateScope;

declare const TextmateInjection: (props: TextmateInjectionProps) => null;

declare class TextmateEventsDelegator extends BaseEventsDelegator<MonacoEventsDefinition, TextmateInjectionProps> {
    scopeName: string[];
    wasmKey: string;
    wireGrammars: Record<string, boolean>;
    providers: Record<string, TextmateProvider>;
    code?: TextmateCodeSet;
    vsRegistry?: Registry;
    vsOnigurumaLib?: Promise<IOnigLib>;
    constructor(props: TextmateInjectionProps);
    get baseUrl(): string | URL;
    get onigurumaWasmUrl(): string | URL;
    get wasmAsset(): {
        key: string;
        url: URL;
        priority: number;
        type: string;
    };
    prepareAssets: ({ preloadAssets, }: MonacoEventsDefinition["prepareAssets"]) => void;
    onAssetVSCodeWasm: ({ task, handle, }: MonacoEventsDefinition["asset"]) => Promise<void>;
    mounting: () => void;
    prepareModel: (params: MonacoEventsDefinition["prepareModel"]) => void;
    createCodeSet: (languageId: string | undefined, params: MonacoEventsDefinition["prepareModel"], provider?: TextmateProvider) => TextmateCodeSet | undefined;
    /**
     * 基于 vscode 新规范，形成正规的 scopeName 和 tmName
     * @param languageId
     * @param extname
     * @param language
     * @param provider
     */
    selectTextmateScope: (languageId: string, extname: string, language?: monaco.languages.ILanguageExtensionPoint, provider?: TextmateProvider) => TextmateScope;
    createModel: ({ monaco, editor }: MonacoEventsDefinition["createModel"]) => void;
    mountEditor: ({ monaco, editor }: MonacoEventsDefinition["mountEditor"]) => void;
    wireVsCodeState: (_monaco: typeof monaco, editor?: monaco.editor.IStandaloneCodeEditor) => void;
    newVsCodeRegistry: (onigLib: Promise<IOnigLib>) => Registry;
}

export { TextmateEventsDelegator, TextmateInjection, isTmSupportLanguage, setupTextmate, tmConfig };
export type { TextmateActiveLanguage, TextmateCodeSet, TextmateFilterCodeSetCallback, TextmateInjectionProps, TextmateProvider, TextmateProviderCallback, TextmateScope };
